$ifndef _HPWM_12F157X_INC_
$define _HPWM_12F157X_INC_
'
' A simple library to control the HPWM peripherals of PIC12F1571 or PIC12F1572 devices
' Written by Les Johnson for the Proton BASIC compiler
'
' The library has several routines:
'
' HPWM1_Period(pPeriod)
' HPWM2_Period(pPeriod)
' HPWM3_Period(pPeriod)
'
' HPWM1_Duty(pDuty)
' HPWM2_Duty(pDuty)
' HPWM3_Duty(pDuty)
'
' HPWM1_Phase(pPhase)
' HPWM2_Phase(pPhase)
' HPWM3_Phase(pPhase)
'
' The library sets up the 3 PWM channels when loaded into a program
'
$if (_device <> _12F1571) And (_device <> _12LF1571) And (_device <> _12F1572) And (_device <> _12LF1572)
    $error "Library only suitable for devices 12F1571, 12LF1571, 12F1572, or 12LF1572"
$else
'
' Define the pins used for HPWM1, HPWM2, and HPWM3
'
$define PWM1_Pin PORTA.1
$define PWM2_Pin PORTA.0
$define PWM3_Pin PORTA.2
'
' Create some high byte PWM SFRs that are "currently" missing from the PPI files
' That allow creation of 16-bit SFRs
' Once they are added to the compiler, the $ifndef/$endif directives will stop duplicate error messages
'
$ifndef _PWM1DCLH
    Symbol PWM1DCLH = PWM1DCH
$endif
$ifndef _PWM1PHLH
    Symbol PWM1PHLH = PWM1PHH
$endif
$ifndef _PWM1PRLH
    Symbol PWM1PRLH = PWM1PRH
$endif
$ifndef _PWM2DCLH
    Symbol PWM2DCLH = PWM2DCH
$endif
$ifndef _PWM2PHLH
    Symbol PWM2PHLH = PWM2PHH
$endif
$ifndef _PWM2PRLH
    Symbol PWM2PRLH = PWM2PRH
$endif
$ifndef _PWM3DCLH
    Symbol PWM3DCLH = PWM3DCH
$endif
$ifndef _PWM3PHLH
    Symbol PWM3PHLH = PWM3PHH
$endif
$ifndef _PWM3PRLH
    Symbol PWM3PRLH = PWM3PRH
$endif
'
' Create some 16-bit PWM SFRs from the low and high SFRs
'
    Dim wPWM1DC_SFR As PWM1DCL.Word
    Dim wPWM1PH_SFR As PWM1PHL.Word
    Dim wPWM1PR_SFR As PWM1PRL.Word

    Dim wPWM2DC_SFR As PWM2DCL.Word
    Dim wPWM2PH_SFR As PWM2PHL.Word
    Dim wPWM2PR_SFR As PWM2PRL.Word

    Dim wPWM3DC_SFR As PWM3DCL.Word
    Dim wPWM3PH_SFR As PWM3PHL.Word
    Dim wPWM3PR_SFR As PWM3PRL.Word

'-------------------------------------------------------------------------------
    GoTo _HPWM_Main_                            ' Jump to the main program segment

'-------------------------------------------------------------------------------
' Adjust the phase of HPWM1
' Input     : pPhase holds the 16-bit phase value
' Output    : None
' Notes     : None
'
$define HPWM1_Phase(pPhase) '
    wPWM1PH_SFR = pPhase    '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the phase of HPWM2
' Input     : pPhase holds the 16-bit phase value
' Output    : None
' Notes     : None
'
$define HPWM2_Phase(pPhase) '
    wPWM2PH_SFR = pPhase    '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the phase of HPWM3
' Input     : pPhase holds the 16-bit phase value
' Output    : None
' Notes     : None
'
$define HPWM3_Phase(pPhase) '
    wPWM3PH_SFR = pPhase    '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the period of HPWM1
' Input     : pPeriod holds the 16-bit period value
' Output    : None
' Notes     : None
'
$define HPWM1_Period(pPeriod) '
    wPWM1PR_SFR = pPeriod     '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the period of HPWM2
' Input     : pPeriod holds the 16-bit period value
' Output    : None
' Notes     : None
'
$define HPWM2_Period(pPeriod) '
    wPWM2PR_SFR = pPeriod     '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the period of HPWM3
' Input     : pPeriod holds the 16-bit period value
' Output    : None
' Notes     : None
'
$define HPWM3_Period(pPeriod) '
    wPWM3PR_SFR = pPeriod     '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the duty of HPWM1
' Input     : pDuty holds the 16-bit duty value
' Output    : None
' Notes     : None
'
$define HPWM1_Duty(pDuty) '
    wPWM1DC_SFR = pDuty   '
    PWM1LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the duty of HPWM2
' Input     : pDuty holds the 16-bit duty value
' Output    : None
' Notes     : None
'
$define HPWM2_Duty(pDuty) '
    wPWM2DC_SFR = pDuty   '
    PWM2LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Adjust the duty of HPWM3
' Input     : pDuty holds the 16-bit duty value
' Output    : None
' Notes     : None
'
$define HPWM3_Duty(pDuty) '
    wPWM3DC_SFR = pDuty   '
    PWM3LDCONbits_LDA = 1

'-------------------------------------------------------------------------------
' Setup the HPWM peripherals
' Input     : None
' Output    : None
' Notes     : None
'
$define HPWM_Setup() GoSub _HPWM_Setup

_HPWM_Setup:
'
' Setup PWM1
'
    PWM1CLKCON = %00000001      ' HFINTOSC clock, no prescaler
    HPWM1_Phase($0000)
    HPWM1_Period($03E7)
    HPWM1_Duty($0000)
    PWM1CON = %11000000         ' Peripheral enable, output enable, normal polarity, standard mode
'
' Setup PWM2
'
    PWM2CLKCON = %00000001      ' HFINTOSC clock, no prescaler
    HPWM2_Phase($0000)
    HPWM2_Period($03E7)
    HPWM2_Duty($0000)
    PWM2CON = %11000000         ' Peripheral enable, output enable, normal polarity, standard mode
'
' Setup PWM3
'
    PWM3CLKCON = %00000001      ' HFINTOSC clock, no prescaler
    HPWM3_Phase($0000)
    HPWM3_Period($03E7)
    HPWM3_Duty($0000)
    PWM3CON = %11000000         ' Peripheral enable, output enable, normal polarity, standard mode
'
' Set the PWM pins to outputs
'
    Low PWM1_Pin
    Low PWM2_Pin
    Low PWM3_Pin
    Return

'-------------------------------------------------------------------------------
' Setup the HPWM peripherals
'
_HPWM_Main_:

    HPWM_Setup()

$endif      ' Device choose
$endif      ' _HPWM_12F157X_INC_
